%ingest_met_data
% Ingests Meteorological data exported from Davis Vantage Pro weather
% station and plots various graphs of parameters vs time
%
% LAST UPDATED by Dr Andrew French. August 2018.

function ingest_met_data

%Choose data file exported from Davis Vantage Pro Weather Link software
[filename, pathname] = uigetfile( {'*.txt',...
    'Text files (*.txt)'},...
    'Select location of Davis Met data file...', ...
    'multiselect', 'off' );
if filename == 0
    return
end

%Start stopwatch
t0 = tic;

%Display loading message to command window
clc;
disp(' ')
disp(' DAVIS VANTAGE PRO WEATHER STATION MATLAB DATA INGESTER ' )
disp(' Dr Andy French. August 2018 ')
disp(' ')
disp( [' ... Ingesting met data from file: ',[pathname,filename],' ...'] )
disp(' ')

%Ingest tab delimited data
d = importdata( [pathname,filename] );
d = d.textdata;
dim = size(d);
N = dim(1)-2;

%Initialize data vectors
t_hours = zeros(1,N);
date_str = cell(1,N);
time_HHMM = cell(1,N);
temp_out = zeros(1,N);
high_temp = zeros(1,N);
low_temp = zeros(1,N);
humidity = zeros(1,N);
dew_point = zeros(1,N);
wind_speed = zeros(1,N);
wind_angle_deg = zeros(1,N);
wind_direction = cell(1,N);
wind_run = zeros(1,N);
high_wind_speed = zeros(1,N);
wind_chill = zeros(1,N);
heat_index = zeros(1,N);
pressure_mb = zeros(1,N);
rain = zeros(1,N);
rain_rate = zeros(1,N);
solar_rad = zeros(1,N);
solar_energy = zeros(1,N);
high_solar_rad = zeros(1,N);
index = zeros(1,N);
UV_dose = zeros(1,N);
UV = zeros(1,N);
High_D_D = zeros(1,N);
Heat_D_D = zeros(1,N);
Cool_temp = zeros(1,N);
in_humidity = zeros(1,N);
in_dew = zeros(1,N);
in_hear = zeros(1,N);
in_EMC = zeros(1,N);
in_density = zeros(1,N);

%Extract data (note two rows of headers)
for n=1:N
    %Print command window note every thousanth line ingested
    if rem(n,1000)==0
        disp( [' Line ', num2str(n),' of ',num2str(N),' ingested' ] )
    end
    
    %Ingest data from file
    try
        t_hours(n) = 24* ( datenum( [ d{ 2+n,1 }, ' ', d{ 2+n,2 } ], 'dd/mm/yyyy HH:MM' ) -...
            datenum( [ d{ 2+1,1 }, ' ', d{ 2+1,2 } ], 'dd/mm/yyyy HH:MM'  ) );
        date_str{n} = d{ 2+n,1 };
        time_HHMM{n} = d{ 2+n,2 };
        temp_out(n) = str2num( d{ 2 + n, 3 } );
        high_temp(n) = str2num( d{ 2 + n, 4 } );
        low_temp(n) =str2num( d{ 2 + n, 5 } );
        humidity(n) =str2num( d{ 2 + n, 6 } );
        dew_point(n) =str2num( d{ 2 + n, 7 } );
        wind_speed(n) =str2num( d{ 2 + n, 8 } );
        wind_angle_deg(n) = wind_angle_func( d{ 2 + n, 9 } );
        wind_direction{n} = d{ 2+n, 9 } ;
        wind_run(n) = str2num( d{ 2 + n, 10 } );
        high_wind_speed(n) = str2num( d{ 2 + n, 11 } );
        wind_chill(n) = str2num( d{ 2 + n, 13 } );
        heat_index(n) = str2num( d{ 2 + n, 14 } );
        pressure_mb(n) = str2num( d{ 2 + n, 17 } );
        rain(n) = str2num( d{ 2 + n, 18 } );
        rain_rate(n) = str2num( d{ 2 + n, 19 } );
        solar_rad(n) = str2num( d{ 2 + n, 20 } );
        solar_energy(n) = str2num( d{ 2 + n, 21 } );
        high_solar_rad(n) = str2num( d{ 2 + n, 22 } );
        index(n) = str2num( d{ 2 + n, 23 } );
        UV_dose(n) = str2num( d{ 2 + n, 24 } );
        UV(n) = str2num( d{ 2 + n, 25 } );
        High_D_D(n) = str2num( d{ 2 + n, 26 } );
        Heat_D_D(n) = str2num( d{ 2 + n, 27 } );
        Cool_temp(n) = str2num( d{ 2 + n, 28 } );
        in_humidity(n) = str2num( d{ 2 + n, 29 } );
        in_dew(n) = str2num( d{ 2 + n, 30 } );
        in_hear(n) = str2num( d{ 2 + n, 31 } );
        in_EMC(n) = str2num( d{ 2 + n, 32 } );
        in_density(n) = str2num( d{ 2 + n, 33 } );
    end
end

%%

%Save data to a .mat file
save( ['met_data ',strrep(filename,'.txt','')],...
    'date_str','t_hours','temp_out','solar_rad','wind_speed','wind_angle_deg',...
    'pressure_mb','humidity','UV','rain_rate' );

%Save data to an Excel file
E = {};
E{1,1} = 'time /hours';
E{1,2} = 'temp /degC';
E{1,3} = 'solar radiation /Wm^-2';
E{1,4} = 'UV index';
E{1,5} = 'wind speed /ms^-1';
E{1,6} = 'wind direction /deg';
E{1,7} = 'Pressure /mb';
E{1,8} = 'humidity %';
E{1,9} = 'rain rate /mm/hour';
for n=1:min( [N,65535] )
    E{ 1+n,1 } = t_hours(n);
    E{ 1+n,2 } = temp_out(n);
    E{ 1+n,3 } = solar_rad(n);
    E{ 1+n,4 } = UV(n); 
    E{ 1+n,5 } = wind_speed(n);
    E{ 1+n,6 } = wind_angle_deg(n);
    E{ 1+n,7 } = pressure_mb(n);
    E{ 1+n,8 } = humidity(n);
    E{ 1+n,9 } = rain_rate(n);
end
if N>65536
    E = E(1:65536,:);
    disp(' Number of data entries exceeds max number of Excel columns.')
    disp(' Data will be truncated to 65,535 entries. ')
end
xlswrite( ['met_data ',strrep(filename,'.txt','.xls')],E )

%Display completion message
disp('')
disp( [' ... File: ',pathname,filename] )
disp( [' ... ingested in ',num2str(toc(t0)),' seconds.'] )
disp( ' ' )

%%

%Function to compute wind direction angle in degrees from wind section code
function a_deg = wind_angle_func(wind_dir)
if strcmp( wind_dir, 'N' )
    a_deg = (360/16)*0;
elseif strcmp( wind_dir, 'NNE' )
    a_deg = (360/16)*1;
elseif strcmp( wind_dir, 'NE' )
    a_deg = (360/16)*2;
elseif strcmp( wind_dir, 'ENE' )
    a_deg = (360/16)*3;
elseif strcmp( wind_dir, 'E' )
    a_deg = (360/16)*4;
elseif strcmp( wind_dir, 'ESE' )
    a_deg = (360/16)*5;
elseif strcmp( wind_dir, 'SE' )
    a_deg = (360/16)*6;
elseif strcmp( wind_dir, 'SSE' )
    a_deg = (360/16)*7;
elseif strcmp( wind_dir, 'S' )
    a_deg = (360/16)*8;
elseif strcmp( wind_dir, 'SSW' )
    a_deg = (360/16)*9;
elseif strcmp( wind_dir, 'SW' )
    a_deg = (360/16)*10;
elseif strcmp( wind_dir, 'WSW' )
    a_deg = (360/16)*11;
elseif strcmp( wind_dir, 'W' )
    a_deg = (360/16)*12;
elseif strcmp( wind_dir, 'WNW' )
    a_deg = (360/16)*13;
elseif strcmp( wind_dir, 'NW' )
    a_deg = (360/16)*14;
elseif strcmp( wind_dir, 'NNW' )
    a_deg = (360/16)*15;
else
    a_deg = NaN;
end

%End of code