%plot_met_data
% Plots met data ingested from Davis Vantage Pro weather station.
%
% LAST UPDATED by Andy French February 2019

function plot_met_data

%Set fontsize
fsize = 18;

%Define ranges for graphs

%Temperature (deg C)
Tmin = -10;
Tmax = 35;

%Humidity (%)
hum_min = 0;
hum_max = 100;

%Pressure (mb)
% https://en.wikipedia.org/wiki/List_of_atmospheric_pressure_records_in_Europe
% For the United Kingdom, the Met Office record the record figures for atmospheric 
% pressure (which are nominally since 1870) as:
%
% Highest air pressure: 31 January 1902, Aberdeen Scotland 1053.6mb.
% Lowest air pressure: 26 January 1884, Ochtertyre Scotland 925.6mb.
Pmin = 950;
Pmax = 1050;

%Solar radiation (W/m^-2). Note max solar irradiance on Earth is 1414 Wm^-2
%https://en.wikipedia.org/wiki/Sunlight
sol_rad_max = 1200;

%UV index
%https://www.epa.gov/sunsafety/uv-index-scale-1
%Extreme is 11. In the Med, 9+ in summer might be typical.
UVmin = 0;
UVmax = 8;

%Wind direction (deg) this will be 0 to 360deg

%Wind speed (m/s)
% Wind speed on the 1946 Beaufort scale is based on the empirical relationship:[5]
% v = 0.836 B^(3/2) (m/s)
% https://en.wikipedia.org/wiki/Beaufort_scale
windmax = 10;

%Rain rate (mm/hr). 'Extreme rain' is more than 50mm/hour. Very heavy rain
%is between 16 and 50mm /hour.
rain_rate_max = 50;

%

%Choose data file exported from Davis Vantage Pro Weather Link software
[filename, pathname] = uigetfile( {'*.mat',...
    'MATLAB data files (*.mat)'},...
    'Select location of MATLAB weather data...', ...
    'multiselect', 'off' );
if filename == 0
    return
end

%Load data
load( [pathname,filename] )

%Define directory name
dirname = [ date_str{1},' to ',date_str{end} ];
dirname = strrep( dirname,'/','-' );
mkdir(dirname);

%Determine end time /days for plots
tmax = ceil( t_hours(end)/24 );

%Plot temperature vs time
fig = figure('color',[1 1 1]);
plot( t_hours/24, temp_out );
ylim([Tmin,Tmax])
xlim([0,tmax])
xlabel('Time /days','fontsize',fsize );
ylabel('Temperature /deg^oC','fontsize',fsize );
title({'Temperature vs time',...
    [ date_str{1},' to ',date_str{end} ]},'fontsize',fsize );
set(gca,'fontsize',fsize );
grid on
print( fig, [dirname,'\temperature vs time.png'],'-dpng','-r300' )
close(fig);

%Plot wind speed vs time
fig = figure('color',[1 1 1]);
plot( t_hours/24, wind_speed );
ylim([0,windmax])
xlim([0,tmax])
xlabel('Time /days','fontsize',fsize );
ylabel('Wind speed /ms^{-1}','fontsize',fsize );
title({'Wind speed vs time',[ date_str{1},' to ',date_str{end} ]},'fontsize',fsize );
set(gca,'fontsize',fsize );
grid on
print( fig, [dirname,'\wind speed vs time.png'],'-dpng','-r300' )
close(fig);

%Plot wind direction vs time
fig = figure('color',[1 1 1]);
plot( t_hours/24, wind_angle_deg );
ylim([0,360])
xlim([0,tmax])
xlabel('Time /days','fontsize',fsize );
ylabel('Wind direction /deg','fontsize',fsize );
title({'Wind direction vs time',[ date_str{1},' to ',date_str{end} ]},'fontsize',fsize );
set(gca,'fontsize',fsize );
grid on
print( fig, [dirname,'\wind angle vs time.png'],'-dpng','-r300' )
close(fig);

%Plot wind speed vs direction wind rose
windrose( t_hours, wind_speed, wind_angle_deg, 2*windmax )
movefile( [pwd,'\wind rose.png'],[pwd,'\',dirname,'\wind rose.png'] )

%Plot pressure vs time
fig = figure('color',[1 1 1]);
plot( t_hours/24, pressure_mb );
ylim([Pmin,Pmax])
xlim([0,tmax])
xlabel('Time /days','fontsize',fsize );
ylabel('Pressure /mb','fontsize',fsize );
title({'Pressure vs time',[ date_str{1},' to ',date_str{end} ]},'fontsize',fsize );
set(gca,'fontsize',fsize );
grid on
print( fig, [dirname,'\Pressure vs time.png'],'-dpng','-r300' )
close(fig);

%Plot solar radiation vs time
fig = figure('color',[1 1 1]);
plot( t_hours/24, solar_rad );
ylim([0,sol_rad_max])
xlim([0,tmax])
xlabel('Time /days','fontsize',fsize );
ylabel('Solar radiation /Wm^{-2}','fontsize',fsize );
title({'Solar radiation vs time',[ date_str{1},' to ',date_str{end} ]},'fontsize',fsize );
set(gca,'fontsize',fsize );
grid on
print( fig, [dirname,'\Solar radiation vs time.png'],'-dpng','-r300' )
close(fig);

%Plot UV index vs time
fig = figure('color',[1 1 1]);
plot( t_hours/24, UV );
ylim([UVmin,UVmax])
xlim([0,tmax])
xlabel('Time /days','fontsize',fsize );
ylabel('UV index','fontsize',fsize );
title({'UV index vs time',[ date_str{1},' to ',date_str{end} ]},'fontsize',fsize );
set(gca,'fontsize',fsize );
grid on
print( fig, [dirname,'\UV index vs time.png'],'-dpng','-r300' )
close(fig);

%Plot humidity vs time
fig = figure('color',[1 1 1]);
plot( t_hours/24, humidity );
ylim([hum_min,hum_max])
xlim([0,tmax])
xlabel('Time /days','fontsize',fsize );
ylabel('Humidity (%)','fontsize',fsize );
title({'Humidity vs time',[ date_str{1},' to ',date_str{end} ]},'fontsize',fsize );
set(gca,'fontsize',fsize );
grid on
print( fig, [dirname,'\Humidity vs time.png'],'-dpng','-r300' )
close(fig);

%Plot rain rate vs time
fig = figure('color',[1 1 1]);
plot( t_hours/24, rain_rate );
ylim([0,rain_rate_max])
xlim([0,tmax])
xlabel('Time /days','fontsize',fsize );
ylabel('Rain rate /mm per hour','fontsize',fsize );
title({'Rain rate vs time',[ date_str{1},' to ',date_str{end} ]},'fontsize',fsize );
set(gca,'fontsize',fsize );
grid on
print( fig, [dirname,'\Rain rate vs time.png'],'-dpng','-r300' )
close(fig);

%End of code