%windrose
% Function that computes a 'wind rose' diagram from vectors of time (t),
% wind speed (v) and wind direction (a) vectors. It is assumed wind
% directions are in sixteenths of a full circle. e.g. (360/16)*7 degrees
% means 'SSE'.
%
% In each direction sector, wind speeds are grouped into N speed bins
% which are colour coded according to frequency.
%
% LAST UPDATED by Andy French August 2018

function windrose( t, v, a, vmax )

%Set number of wind speed bins
N = 32;

%Fontsize for graphs
fsize = 18;

%

%Find smallest time interval
dt = min(diff(t));

%Remove NaN values
inan = find( isnan(t) | isnan(v) | isnan(a) );
t(inan) = [];
v(inan) = [];
a(inan) = [];

%Interpolate v,a at equal time intervals dt
torig = t;
t = t(1) : dt : t(end);
v = interp1( torig, v, t );
a = interp1( torig, a, t );

%Define speed (m/s) bin edges
SBE = linspace(0,vmax,N+1);

%Define wind direction (deg) edges
WDE = 0 : 360/16 : 360*15/16;

%Determine wind direction bin number
Wj = whichbin( WDE, a );
    
%Determine wind speed bin number
Wi = whichbin( SBE, v );

%Step through data and filter by wind direction and wind speed, add
%frequencies to a matrix W
W = zeros(N+1,16);
for k=1:length(t)
    %Add to W
    W( Wi(k),Wj(k) ) = W( Wi(k),Wj(k) ) + 1;
end

%Determine log of max frequency, to correspond to end of colour scale
CSmax = log10( max( max(W) ) );

%Initialise figure and axes
fig = figure('name','wind rose','color',[1 1 1]);
ax = axes('nextplot','add');
interp_colormap(500);

%Step through elements of W and plot a sector, coloured by quantity
for i=1:N-1
    for j=1:16
        
        %Determine sector patch x,y coordinates
        r0 = SBE(i);
        r1 = SBE(i) + vmax/N;
        a0 = WDE(j);
        a1 = WDE(j) + 360/16;
        a = (pi/180)*linspace(a0,a1,500);
        x = [r0*sin(a),r1*sin( fliplr(a) ), r0*sin( a(1) ) ];
        y = [r0*cos(a),r1*cos( fliplr(a) ), r0*cos( a(1) ) ];
        
        %Plot patch
        if W(i,j)>0
            p = patch( x,y, RGB_from_n( log10( W(i,j) ), CSmax ),...
                'facealpha', 0.8, 'edgecolor','none' );
        end
    end
end

%Graph annotations
axis equal
axis off
caxis([ 0, CSmax] );
colorbar('fontsize',fsize)
title( { ['Wind speed vs angle. Max speed = ',num2str(vmax),'m/s'],...
    ['Max colour means frequency of 10^{ ',num2str(CSmax,2),'} = ',...
    num2str(10^CSmax) ] },'fontsize',fsize )
set(gcf,'units','normalized')
set(gcf,'position',[0,0,1,1])
print( fig, 'wind rose.png', '-dpng', '-r300' );
close(fig);

%%

%Function which determines bin index of x given vector of bin edges be.
function bi = whichbin( be, x )

%Step through x and determine bin
bi = zeros(size(x));
for i=1:length(x)
    for j=2:length(be)
        if ( x(i)>=be(j-1) ) && ( x(i) < be(j) )
            bi(i) = j-1;
        end
    end
end

%Any unallocated bins must be the final one (i.e. x >= than final edge)
bi( bi==0 ) = length(be);

%%

%RBG colour from value n
function RGB = RGB_from_n( n,nmax )

%Get current colourmap
map = colormap;

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield RGB
RGB = [0,0,0];
x = linspace(0,1,dim(1));
RGB(1) = interp1( x, R, n/nmax );
RGB(2) = interp1( x, G, n/nmax );
RGB(3) = interp1( x, B, n/nmax );

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap( N )

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code